
# q-expansion of newform 2304.2.a.p, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2304
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [4], [0], [0], [-4], [2], [0], [0], [4], [0], [12], [10], [0], [0], [4], [0], [12], [0], [0], [-6], [0], [0], [-10], [-18], [20], [0], [0], [-20], [14], [0], [0], [-22], [0], [20], [0], [12], [0], [0], [4], [0], [-20], [0], [14], [-28], [0], [0], [0], [0], [-4], [-26], [0], [30], [0], [-2], [0], [20], [0], [28], [-10], [0], [4], [0], [0], [-26], [-28], [0], [18], [0], [-36], [-34], [0], [0], [-36], [0], [0], [20], [12], [2], [6], [0], [28], [0], [-34], [0], [0], [-14], [-42], [20], [0], [0], [0], [0], [0], [0], [0], [-44], [-22], [0], [-20], [0], [-28], [0], [26], [0], [2], [0], [46], [0], [10], [0], [-36], [38], [0], [0], [50], [0], [0], [-44], [0], [12], [46], [52], [0], [0], [52], [44], [0], [0], [-4], [0], [0], [0], [-52], [-38], [-50], [-44], [0], [52], [10], [0], [-28], [0], [0], [-20], [0], [-36], [58], [0], [0], [12], [-50], [0], [0], [0], [0], [0], [-46], [-38], [20], [0], [26], [0], [0], [-62], [0], [0], [12], [30], [-44], [0], [60], [0], [-6], [0], [-10], [0], [20], [0], [60], [0], [0], [-4], [-58], [0], [-44], [-52], [0], [54], [0], [66], [0], [0], [68], [0], [-26], [50], [44], [62], [0], [4], [0], [-68], [-30], [0], [68], [0], [0], [70], [0], [-2], [52], [0], [0], [0], [-10], [0], [-62], [0], [4], [-68], [0], [50], [0], [0], [60], [-74], [0], [0], [0], [76], [0], [0], [70], [0], [0], [-66], [-76], [0], [0], [0], [0], [0], [-36], [46], [0], [0], [0], [0], [0], [-68], [-2], [0], [-6], [-76], [0], [-20], [0], [52], [38], [0], [0], [76], [-36], [82], [0], [-44], [22], [0], [-76], [60], [0], [-54], [0], [-78], [0], [0], [-84], [-70], [0], [0], [0], [0], [60], [0], [0], [-66], [-28], [0], [-34], [52], [0], [86], [0], [-84], [20], [0], [-76], [0], [0], [86], [68], [0], [0], [0], [-18], [0], [-4], [0], [-84], [0], [-76], [82], [0], [0], [90], [0], [0], [66], [46], [0], [-58], [-92], [0], [74], [30], [0], [0], [0], [4], [28], [-92], [0], [0], [0], [0], [60], [94], [90], [0], [-84], [-38], [20], [0], [-44], [0], [92], [0], [0], [52], [0], [-42], [68], [0], [-84], [-74], [0], [0], [98], [0], [12], [-58], [0], [0], [0], [26], [-92], [0], [70], [0], [0], [0], [100], [0], [92], [0], [0], [34], [-94], [102], [100], [-86], [0], [-98], [0], [0], [0], [-68], [0], [0], [66], [-44], [0], [0], [0], [6], [0], [10], [0], [-92], [60], [-14], [0], [58], [100], [0], [28], [98], [0], [0], [-46], [68], [0], [0], [-102], [100], [0], [0], [-62], [0], [20], [108], [0], [0], [-106], [-92], [0], [-74], [0], [0]]
